/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* core03_db.h was generated from the DDL file core03_db.ddl by the ddlp.
 * utility.
 */
#include "core03_db.h"

/* core03_db_dbd.h was generated from the DDL file core03_db.ddl by the 
 * ddlp utility.  It is required to use the d_open_ptr API
 */
#include "core03_db_dbd.h"

/* EXAMPLE - Core for C */
/* Each time you run this example:
 *   1) The database is initialized (all existing contents removed)
 *   2) Several new artists are added to the database
 *   3) For each artist a couple albums are added to the database
 *   4) A list of each artist and their albums are displayed
 *
 * The DDL (Database Definition Language) is in core03_db.ddl. The database
 * schema is embedded into the application and the database will be created 
 * the first time the application is run.
 *
 * The following RDM Functions are illustrated:
 *      d_opentask/d_closetask
 *      d_open_ptr/d_close
 *      d_initialize
 *      d_fillnew
 *      d_setor
 *      d_connect
 *      d_recfrst/d_recnext
 *      d_findfm/d_findnm
 *      d_recread
 *      d_errorstr
 *      d_errorname
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Because the database is opened in exclusive mode lock and transactions
 * are not required.  Future core examples will use shared mode and will
 * demonstrate the use of transactions and locks.
 *
 * Each of these functions returns an integer status code, where the
 * value S_OKAY indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * The actual database will be stored in a directory named 'core03_db' in the
 * project directory.
 */

const char *doors_albums[] = 
{
    "The Doors",
    "Strange Days",
    "Waiting for the Sun"
};

const char *stones_albums[] =
{
    "The Rolling Stones",
    "Out of Our Heads",
    "Beggars Banquet",
    "Tattoo You"
};

const char * nirvana_albums[] =
{
    "Bleach",
    "Nevermind",
    "In Utero"
};

/* =======================================================================
*/
int32_t main(void)
{
    DB_TASK       *task;
    int32_t        status;
    uint16_t       ii;
    struct artist  artist_rec;
    struct album   album_rec;

    /* create the context for the database operations */
    d_opentask(&task);

    /* open the database for exclusive access, this will create the database
     * dictionary (core03_db.dbd file if it does not already exist.
     */
    status = d_open_ptr("core03_db", "x", &core03_db_dbd, sizeof(core03_db_dbd), task);
    if (status != S_OKAY)
    {
        fprintf (stderr, "Exclusive open failed, error = %s: %s\n", d_errorname(status), d_errorstr(status) );
        d_closetask (task);
        return (1);
    }

    /* Initializing the database will create the data files if they do not exist.
     * If the the data files already exist they will be removed and empty
     * files will be created.
     */
    d_initialize(task, CURR_DB);
    
    /* Create an artist record instance, set it as the owner of the 
     * ARTIST_ALBUM set */
    strcpy(artist_rec.name, "The Doors");
    d_fillnew(ARTIST, &artist_rec, task, CURR_DB);
    d_setor(ARTIST_ALBUM, task, CURR_DB);

    /* Add associated albums */
    for(ii = 0; ii < sizeof(doors_albums)/sizeof(doors_albums[0]); ii++)
    {
        strcpy(album_rec.title, doors_albums[ii]);
        d_fillnew(ALBUM, &album_rec, task, CURR_DB);
        d_connect(ARTIST_ALBUM, task, CURR_DB);
    }

    /* Create an artist record instance, set it as the owner of the 
     * ARTIST_ALBUM set */
    strcpy(artist_rec.name, "The Rolling Stones");
    d_fillnew(ARTIST, &artist_rec, task, CURR_DB);
    d_setor(ARTIST_ALBUM, task, CURR_DB);

    /* Add associated albums */
    for(ii = 0; ii < sizeof(stones_albums)/sizeof(stones_albums[0]); ii++)
    {
        strcpy(album_rec.title, stones_albums[ii]);
        d_fillnew(ALBUM, &album_rec, task, CURR_DB);
        d_connect(ARTIST_ALBUM, task, CURR_DB);
    }

    /* Create an artist record instance, set it as the owner of the 
     * ARTIST_ALBUM set */
    strcpy(artist_rec.name, "Nirvana");
    d_fillnew(ARTIST, &artist_rec, task, CURR_DB);
    d_setor(ARTIST_ALBUM, task, CURR_DB);

    /* Add associated albums */
    for(ii = 0; ii < sizeof(nirvana_albums)/sizeof(nirvana_albums[0]); ii++)
    {
        strcpy(album_rec.title, nirvana_albums[ii]);
        d_fillnew(ALBUM, &album_rec, task, CURR_DB);
        d_connect(ARTIST_ALBUM, task, CURR_DB);
    }

    /* Loop through each artist record and diplay the albums they are
       associated with
     */
    printf("\nList all artists and their albums\n");
    for (status = d_recfrst (ARTIST, task, CURR_DB);
         status == S_OKAY;
         status = d_recnext (task, CURR_DB))
    {
        /* Establish the current student instance as the owner of the 
         * ARTIST_ALBUM set
         */
        d_setor(ARTIST_ALBUM, task, CURR_DB);

        /* Read and display the current artist record */
        d_recread(&artist_rec, task, CURR_DB);
        printf("\n%s\n", artist_rec.name);

        /* Find all of the albums records related to this artist */
        for (status = d_findfm (ARTIST_ALBUM, task, CURR_DB);
             status == S_OKAY;
             status = d_findnm (ARTIST_ALBUM, task, CURR_DB))
        {
            /* Read and display the album information  */
            status = d_recread(&album_rec, task, CURR_DB);
            printf("\t%s\n", album_rec.title);
        }
    }
    
    /* close the database */
    d_close (task);
    d_closetask (task);
    
    printf ("\nPress <Enter> to continue . . .\n");
    getchar ();
}
